<?php

/**
 * Plugin Name: pd Woo Tracking Order
 * Description: You can set the custom Woocommerce Order Status and can able to add a Tracking ID of your carrier service provider with a nice front end user interface of tracking system.
 * Plugin URI: https://proficientdesigners.in/creations/pd-woo-tracking-order/
 * Author: Proficient Designers
 * Author URI: https://proficientdesigners.com
 * Version: 2.0.3
 * WC requires at least: 3.4.2
 * WC tested up to: 4.9.1
 * License: GPL2 or later
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: pd-woo-tracking-order
 * Network: false
 */

defined('ABSPATH') || exit;

/**
 * Default Variables
 */
$plugin_uri		= basename(plugin_dir_path(__FILE__));
$plugin_name 	= 'pd Woo Tracking Order';

/**
 * Version checks
 */
global $wp_version;

if (version_compare($wp_version, '4.0', '<')) {
	wp_die(
		'Sorry, <b>' . $plugin_name . '</b> plugin requires WordPress 4.0 or newer. <p><a class="button" href="http://codex.wordpress.org/Upgrading_WordPress">Please update!</a></p>',
		'Warning !!',
		['back_link' => true]
	);
}

if (version_compare(phpversion(), '5.6', '<')) {
	wp_die(
		'Sorry, <b>' . $plugin_name . '</b> plugin requires php version 5.6 or above',
		'Warning !!',
		['back_link' => true]
	);
}

/**
 * Include all classes
 */
require_once plugin_dir_path(__FILE__) . 'class/class-front.php';
require_once plugin_dir_path(__FILE__) . 'class/class-email.php';
require_once plugin_dir_path(__FILE__) . 'class/class-admin.php';

/**
 * Intiating main class for pd-woo-tracking-order
 */
if (!class_exists('pdWooTrackingOrder')) {

	class pdWooTrackingOrder
	{

		private $plugin_name;

		public function __construct($plugin_name)
		{

			$this->plugin_name = $plugin_name;

			register_activation_hook(__FILE__, [$this, 'activate']);
			add_action('admin_init', [$this, 'detect_woo_deactivation'], 10);
			add_action('plugins_loaded', [$this, 'on_load']);

			if (isset($_GET['post_type']) && $_GET['post_type'] === 'pd_tracking_order') {
				add_action('pre_get_posts', [$this, 'set_order_status_menu_order']);
			}
		}

		/**
		 * Check if WooCommerce is active
		 */
		public function activate()
		{

			if (!in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {

				wp_die(
					'Sorry, <b>' . $this->plugin_name . '</b> plugin requires <b><a href="https://wordpress.org/plugins/woocommerce/">Woocommerce 3.0</a></b> or higher...',
					'Warning !!',
					['back_link' => true]
				);
			}
		}

		/**
		 * Deactivate this plugin when Woocommerce is deactivated automatically and if its
		 * deactivated show wp a admin notice this plugin is deactivated
		 */
		public function detect_woo_deactivation()
		{

			if (!in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {

				deactivate_plugins(plugin_basename(__FILE__));

				add_action('admin_notices', function () {
					$class = 'notice notice-error dismissible';
					$message = 'It seems <b>Woocommerce</b> is deactivated and so <b><a>pd Woo Tracking Order</a></b> also got deactivated too, but your settings remains saved !!';
					printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($class), $message);
				}, 10);
			}
		}

		/**
		 * This func loads Admin & Front classes
		 */
		public function on_load()
		{

			if (class_exists('pd_Woo_Tracking_Order_Front')) {
				new pd_Woo_Tracking_Order_Front();
			}

			if (!current_user_can('manage_woocommerce'))
				return;

			add_action('admin_init', [$this, 'defaults']);
			add_filter('plugin_action_links', [$this, 'add_settings_link'], 10, 5);

			if (class_exists('pd_Woo_Tracking_Order_Admin')) {
				global $plugin_uri;
				new pd_Woo_Tracking_Order_Admin($plugin_uri);
			}

			$inc_at_once = [
				'admin/custom-column/*.php',
				'ajax/*.php'
			];

			foreach ($inc_at_once as $route) {
				foreach (glob(plugin_dir_path(__FILE__) . $route) as $file) {
					require_once $file;
				}
			}
		}

		/**
		 * Default settings are added while the plugin is activated
		 */
		public function defaults()
		{
			/**
			 * Global Settings
			 */
			$fields = [
				'tracker_title'				=> 'Track your order #{{order_id}} which is {{order_status}}',
				'thankyou_page_visibility' 	=> 'off',
				'order_page_visibility'		=> 'off',
				'tracker_line_color' 		=> '#0dcc0d',
				'tracker_line_width' 		=> '0.26',
				'send_email'				=> 'off',
				'default_carrier'			=> ''
			];

			add_option('_pd_woo_tracking_order_settings', $fields);

			/**
			 * Adding default Woo Statuses
			 */
			$this->add_default_statuses();
		}

		/**
		 * pre_get_posts set pd_tracking_order cpt to orderby menu_order
		 */
		public function set_order_status_menu_order($query)
		{

			if (is_admin()) {
				$query->set('order', 'asc');
				$query->set('orderby', 'menu_order');
			}
		}

		/**
		 * Default statuses are added while the plugin is activated
		 */
		public function add_default_statuses()
		{

			if (!current_user_can('edit_posts'))
				return;

			if (get_option('_pd_woo_default_order_statuses_init', false) == 1)
				return;

			$statuses = wc_get_order_statuses();
			$menu_order = 0;

			foreach ($statuses as $status) {

				if (post_exists($status, '', '', 'pd_tracking_order') !== 0) return;

				$postarr = [
					'post_type' => 'pd_tracking_order',
					'post_status' => 'publish',
					'post_title' => $status,
					'menu_order' => $menu_order
				];

				wp_insert_post($postarr, false);

				$menu_order++;
			}

			update_option('_pd_woo_default_order_statuses_init', 1, null);
		}

		/**
		 * Plugin settings and documentation links are added
		 * @param [array] $actions
		 * @param [base_dir] $plugin_file
		 */
		public function add_settings_link($actions, $plugin_file)
		{

			static $plugin;

			if (!isset($plugin))

				$plugin = plugin_basename(__FILE__);

			if ($plugin == $plugin_file) {

				$docs_link 	= ['docs' => '<a href="https://proficientdesigners.in/creations/pd-woo-tracking-order/" target="_blank">' . __('Docs', 'pd-woo-tracking-order') . '</a>'];
				$settings 	= ['settings' => '<a href="edit.php?post_type=pd_tracking_order">' . __('Get Started', 'pd-woo-tracking-order') . '</a>'];

				$actions = array_merge($docs_link, $actions);
				$actions = array_merge($settings, $actions);
			}

			return $actions;
		}
	}

	new pdWooTrackingOrder($plugin_name);
}
